<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

use DB;
use Schema;
use DateTime;
use ReflectionClass;
use App\Models\Inflect;

class ExtendModel extends Model
{
    const paginate = 30;
    const session_alert_status	=	'alert-status';
    const session_alert_icon 	=	'alert-icon';
    const session_alert_msg 	=	'alert-message';
    const global_exclude_key = ['_token','request_type'];

    /*
    |--------------------------------------------------------------------------
    | Here to manage of application settings
    |--------------------------------------------------------------------------
    |
    */

    public static function set_connection($connection_string){

        //return DB::connection($connection_string);
    }

    public static function app_title()
    {
        return 'Leaf Extend';
    }

    public static function trans($value)
    {
    	return $value;
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of set & get mutators
    |--------------------------------------------------------------------------
    |
    */
    public function getDayInWord($day)
    {
        $day = (int) $day;
        $st_days = array(1,21,31);
        $nd_days = array(2,12,22);
        $rd_days = array(3,13,23);
        switch($day)
        {
            case (in_array($day , $st_days)):
            return $day.' st';
            break;

            case (in_array($day , $nd_days)):
            return $day.' nd';
            break;

            case (in_array($day , $rd_days)):
            return $day.' rd';
            break;

            default:
            return $day.' th';
            break;

        }
    }

    public static function getGlobalExcludeKeys()
    {
        return ['_token','request_type','language_listing'];
    }

    public static function getStateKeys()
    {
        return ['status'];
    }

    public static function getPhotoKeys()
    {
        return ['logo_photo_path','cover_photo_path','favicon_photo_path'];
    }
    

    public static function getArrayKeys()
    {
        return ['category_ids', 'gallery_photos','contents','social_medias','working_days'];
    }

    public static function getSingleTableArrayKeys()
    {
        return ['title','share_title','share_description','content'];
    }
    
    public function getDouble($value)
    {
        $value = str_replace(',', '', ($value ? $value:0));
        return number_format($value, 2,'.','');
    }

    public function setDouble($value)
    {
        $value = str_replace(',', '', $value);
        return number_format((float)$value, 2,'.','');
    }

    public function getDate($value)
    {
        if ($value) {
            return date('d-m-Y', strtotime($value));
        }
    }

    public function setDate($value)
    {
        if ($value) {
            return date('Y-m-d', strtotime($value));
        }
    }

    public function setTime($value)
    {
        if ($value) {
            return date('H:i:s', strtotime($value));
        }
    }

    public function convertDateToString($col)
    {
        $date = new DateTime($this->$col);
        $this->$col = $date->format("F j, Y");
    }

    public static function getByKey($key,$needle)
    {
        return static::where($key,'=',$needle)->get();
    }

    public static function getByPage($page,$no_per_page)
    {
     
        $skip_before = $page == 1 ? 0 : $page * $no_per_page;
        $return = static::skip($skip_before)
                        ->take($no_per_page)
                        ->get();


        return $return;
    }


    /*
    |--------------------------------------------------------------------------
    | Here to manage of scope
    |--------------------------------------------------------------------------
    |
    */

    public function scopeOfAvailable($query, $col, $bool)
    {
        return $query->where($col, '=', $bool);
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of displayed data
    |--------------------------------------------------------------------------
    |
    */

    public function encrypt($value)
    {
        return encrypt($value);
    }

    public function decrypt($value)
    {
        return decrypt($value);
    }

    public function display_substr($value,$length=null)
    {
        return substr($value, (isset($length) ? $length : 50));
    }

    public function display_relation_class($parent, $class)
    {
        $parent = str_replace('_id', '', $parent);
        return ($this->$parent ? $this->$parent->$class():'');
    }

    public function display_relation_child($parent, $class)
    {
        $parent = str_replace('_id', '', $parent);
        return ($this->$parent ? $this->$parent->$class:[]);
    }

    public function display_relationed($parent, $col)
    {
        $parent = str_replace('_id', '', $parent);
        return ($this->$parent ? $this->$parent->$col:'');
    }

    public function display_answer_string($col)
    {
    	return ($this->$col ? Language::trans('Yes'):Language::trans('No'));
    }

    public function display_status_string($col)
    {
    	return ($col ? Language::trans('Enabled'):Language::trans('Disabled'));
    }
	
	public function display_room_type_string($col){

        $roomType = "";

        if($this->$col == 'single'){
            $roomType = Setting::SINGLE_ROOM;
        }else if($this->$col == 'twin'){
            $roomType = Setting::TWIN_ROOM;
        }

        return Language::trans($roomType) ;
    }

    public function display_date($value)
    {
    	if ($value) {
    		return date('d-m-Y', strtotime($value));
    	}
    }

    public static function status_true_word()
    {
        return Language::trans('Active');
    }

    public static function status_false_word()
    {
        return Language::trans('Inactive');
    }

    public static function answer_true_word()
    {
        return Language::trans('Yes');
    }

    public static function answer_false_word()
    {
        return Language::trans('No');
    }

    public static function get_module_name($class_name){

        $reflectionClass = new ReflectionClass($class_name);
        $reflectionProperty = $reflectionClass->getProperty('table');
        $reflectionProperty->setAccessible(true);
        $table_name = Inflect::singularize($reflectionProperty->getValue(new $class_name));

        return ucwords(str_replace('_', " ", $table_name));
    }

    public function display_datetime_wo_second($value)
    {
        if ($value) {
            return date('d-m-Y H:i', strtotime($value));
        }
    }

    public function display_datetime($value)
    {
    	if ($value) {
    		return date('d-m-Y H:i:s', strtotime($value));
    	}
    }

    public function display_time_wo_second($value)
    {
    	if ($value) {
    		return date('H:i', strtotime($value));
    	}
    }

    public function display_time($value)
    {
    	if ($value) {
    		return date('H:i:s', strtotime($value));
    	}
    }

    public function three_month_pass()
    {
        return date('m-Y', strtotime('-3 month'));
    }

    public function last_month()
    {
        return date('m-Y', strtotime('-1 month'));
    }

    public static function previous_one_year_combobox()
    {
        for ($i=12; $i>=0; $i--) { 
            $string = date('m-Y', strtotime('-'.$i.' month'));
            $return[(string) $string] = (string) $string;
        }
        return $return;
    }

    public static function next_one_year_combobox()
    {
        for ($i=0; $i<=12; $i++) { 
            $string = date('m-Y', strtotime('+'.$i.' month'));
            $return[(string) $string] = (string) $string;
        }
        return $return;
    }

    public static function month_year_combobox($total_year=1)
    {
        $total_months = $total_year * 12;
        $start = 0 - $total_months/3;
        $end = $total_months/3;
        for ($i=$start; $i<=$end; $i++) { 
            $action = $i < 0 ? '- '.$i : '+ '.$i;
            $string = date('m-Y', strtotime($action.' month'));
            $return[(string) $string] = (string) $string;
        }
        return $return;
    }

}
