<?php

namespace App\Models;

use DB;
use Auth;
use Schema;
use Validator;
use Request;
use Image;


use Illuminate\Database\Eloquent\Builder;

class Faq extends ExtendModel
{
    protected $table = 'faqs';
    public $timestamps = true;
    protected $listing_except_columns = ['hashtags','is_suspend','country_id','google_map_location','leaf_group_id','ky_group_id','sub_category_id','creator_facebook_url','user_facebook_app_id','user_facebook_page_id','youtube_links','created_by','updated_by','cancelled_by','created_at','updated_at'];



    protected $guarded = [];

    /*
    |--------------------------------------------------------------------------
    | Here to manage of relationships
    |--------------------------------------------------------------------------
    |
    */

    public function creator()
    {
        return $this->belongsTo('App\Models\User', 'created_by');
    }

    public function updator()
    {
        return $this->belongsTo('App\Models\User', 'updated_by');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of scope
    |--------------------------------------------------------------------------
    |
    */

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('owned_by', function (Builder $builder) {
            $builder->where('status', '=', true);
        });
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of data's
    |--------------------------------------------------------------------------
    |
    */

    public static function combobox($state_id=null)
    {
        return static::ofAvailable('status',true)
                                ->where('state_id','=',$state_id)
                                ->pluck('name','id')
                                ->prepend(Language::trans('Please select Faq...'), '');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of index listing displayed
    |--------------------------------------------------------------------------
    |
    */

    public function table_cols()
    {
        $except = $this->listing_except_columns;

        return array_diff(Schema::getColumnListing($this->table), $except);
    }

    public function listing_header()
    {
        return array_diff($this->table_cols(), $this->listing_except_columns);
    }

    public function scopeListing($query) 
    {
        return $query->select(array_diff($this->table_cols(), $this->listing_except_columns));
    }

    /*
    |--------------------------------------------------------------------------
    | Setter and getter
    |--------------------------------------------------------------------------
    |
    */
    public static function getRecentFaq($date = false)
    {
        $date = $date != false ? $date : date('Y-m-d', strtotime('now'));
        $return = static::where('held_at','<=', $date)->get();


        return $return;
    }


    /*
    |--------------------------------------------------------------------------
    | Here to manage of validation & save form
    |--------------------------------------------------------------------------
    |
    */

    public function validate_form($input)
    {
        $rules = [
                    'state_id'  =>  'required',
                    'code'      =>  'required|unique:faqs,code,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    'name'      =>  'required|unique:faqs,name,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    ];

        if ($this->id) {
            $rules['code'] = 'required|unique:faqs,code,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
            $rules['name'] = 'required|unique:faqs,name,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
        }

        $validator = Validator::make($input, $rules);

        if ($validator->fails()) {
            return $validator;
        }
        return false;
    }

    public static function getLatest($offset=false)
    {
        $return = static::select('*')
                        ->orderBy('sort_order','asc');
        if($offset != false)
        {
            $return = $return->skip(0)->take($offset);
        }

        return $return->get();
    }
    
    const excluded_keys = [];
    const child_key = ['contents'];
    const array_keys = ['title','content'];
      
    public function save_form($input)
    {
        //dd($input);
        //dd( static::getGlobalExcludeKeys());
        $inputs = isset($input[$input['request_type']]) ? $input[$input['request_type']] : false;
        //dd($input);
        DB::beginTransaction();
        try {

                    $default_keys = Company::getSaveModelDefaultKeys();

                    foreach($inputs as $i_index => $input)
                    {

                            $model = isset($input['id']) ?   ( $input['id'] != null ? static::find($input['id']) : new static() ) : new static();
                            foreach ($input as $key => $value) 
                            {
                                if (!in_array($key , static::getStateKeys()) && !in_array($key , static::excluded_keys) && !in_array($key , static::getGlobalExcludeKeys()) && !in_array($key , static::child_key) && !in_array($key , static::array_keys)) {

                                   if(is_array($value))
                                   {
                                        continue;
                                   }
                                  
                                   $model[$key] = (string) $value;
                                   
                                    
                                }else if(in_array($key, static::excluded_keys))
                                {
                                    //$model[$key = $value == 'on' ? true : false;

                                }else if(in_array($key, static::array_keys))
                                {

                                    $model[$key] = json_encode($value);
                                }
                                //End special key check
                            }

                                
                            
                            foreach($default_keys as $d_key => $d_value){
                                 $model[$d_key] = $d_value;
                             }
                            
                            if (!$model['id']) {
                                $model['created_by']       =   Auth::id() ? Auth::id():0;
                                $model['updated_by']       =   0;
                                $model['leaf_group_id']    =   Company::get_group_id();
                                $model['updated_by']       =   Auth::id() ? Auth::id():0;
                            } else {
                                $model['updated_by']       =   Auth::id() ? Auth::id():0;
                            }
                     
                            foreach($model->attributes as $key => $value){
                         
                                if(strpos($key,'_id') !== false)
                                {
                                     //dd($key);
                                    if($value == '' || $value == null){
                                        
                                        $model[$key] = 0;
                                    }
                                }
                            }
                          
                            $model->save();

                }
        } catch (Exception $e) {
            //dd($e);
            throw $e;
            DB::rollBack();
        }
        DB::commit();
    }
}
